SystemOrganization addCategory: #'EjerciciosPoo-TusLibros'!

Error subclass: #BookNotInCatalogException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

Error subclass: #NotAValidQuantityException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

Object subclass: #Cart
	instanceVariableNames: 'contents catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/13/2012 21:28'!
withCatalog: aCollection 
	^Cart new initializeCatalogOn: aCollection! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/13/2012 21:34'!
add: anISBN withQuantity: aNumber
	
	aNumber > 0 ifFalse: [NotAValidQuantityException new signal].
	(self isAListedISBN: anISBN) ifFalse: [BookNotInCatalogException new signal.].
	contents add: anISBN withOccurrences: aNumber.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/13/2012 21:29'!
initializeCatalogOn: aCollection 
	catalog := aCollection.
	contents := Bag new.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/13/2012 21:30'!
isAListedISBN: anISBN  
	
	^catalog includes: anISBN.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/13/2012 21:32'!
isEmpty
	^contents isEmpty! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/12/2012 19:11'!
listContents
	^ contents.! !

Object subclass: #Cashier
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 22:14'!
checkCartIsNotEmpty: aCart
	aCart isEmpty
		ifTrue: [ Error signal: 'Cart should not be empty' ]! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 22:03'!
checkout: aCart withCreditCard: aCreditCard
	self checkCartIsNotEmpty: aCart.
	self validateCreditCard: aCreditCard.! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 22:40'!
today 
	|currentDate|
	currentDate := Date current.
	^ MonthOfYear onMonthNumber: currentDate month index of: currentDate year ! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 22:47'!
validateCreditCard: aCreditCard 
	(aCreditCard isExpiredOn: self today) ifTrue:[Error signal: 'Credit card is expired']. 
	(aCreditCard owner size > 30) ifTrue:[Error signal: 'Owner name is too long']. ! !

TestCase subclass: #TusLibrosTest
	instanceVariableNames: 'marketplace'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 22:12'!
cartWithCatalog: aCollectionOfBooks
	
	^Cart withCatalog: aCollectionOfBooks.! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 21:48'!
creditCard
	^ CreditCard ownedBy: 'Juan' identifiedBy: 123456 expiresOn: (MonthOfYear januaryOf: Date current year + 1)! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 22:12'!
emptyCart
	
	 ^self cartWithCatalog: #('book1' 'book2' 'book3')! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 22:26'!
expiredDate
	^MonthOfYear januaryOf: Date current year - 1! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 21:57'!
fullCart
	^ self emptyCart
		add: 'book1' withQuantity: 2;
		add: 'book2' withQuantity: 1! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 21:50'!
test01CartsAreCreatedEmpty
	self assert: (self emptyCart isEmpty)! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 21:50'!
test02CantAddBookNotInCatalogToCart
	| cart |
	cart := self emptyCart.
	self should: [ cart add: 'notBook' withQuantity: 1 ] raise: BookNotInCatalogException.
	self assert: cart isEmpty.! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 21:50'!
test03CantAddZeroOrNegativeNumberOfBooks
	| cart |
	cart := self emptyCart.
	self should: [ cart add: 'book1' withQuantity: 0 ] raise: NotAValidQuantityException.
	self assert: cart listContents isEmpty.
	self should: [ cart add: 'book1' withQuantity: -1 ] raise: NotAValidQuantityException.
	self assert: cart listContents isEmpty! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 21:50'!
test04ACanAddBookInCatalogue
	| cart list |
	cart := self emptyCart.
	cart add: 'book1' withQuantity: 1.
	list := cart listContents.
	self assert: (list occurrencesOf: 'book1') = 1! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 21:50'!
test04BCanAddMultipleBooksMultipleTimes
	| cart list |
	cart := self emptyCart.
	cart add: 'book1' withQuantity: 1.
	cart add: 'book1' withQuantity: 3.
	cart add: 'book2' withQuantity: 2.
	list := cart listContents.
	self assert: (list occurrencesOf: 'book1') = 4.
	self assert: (list occurrencesOf: 'book2') = 2! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 21:50'!
test05CantCheckoutEmptyCart

	self
		should: [ Cashier checkout: self emptyCart withCreditCard: self creditCard ]
		raise: Error
		withExceptionDo: [ :anError | anError messageText = 'Cannot checkout empty cart' ]
	! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 22:27'!
test06CantCheckoutWithExpiredCreditCard
	self
		should: [ 
			Cashier
				checkout: self fullCart
				withCreditCard: (CreditCard ownedBy: 'Juan' identifiedBy: 1222 expiresOn: self expiredDate) ]
		raise: Error
		withExceptionDo: [ :anError | anError messageText = 'Credit card is expired' ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 22:49'!
test07CantCheckoutWithNameTooLong
	self
		should: [ 
			Cashier
				checkout: self fullCart
				withCreditCard:
					(CreditCard ownedBy: 'JuanJuanJuanJuanJuanJuanJuanJuan' identifiedBy: 1222 expiresOn: self expiredDate) ]
		raise: Error
		withExceptionDo: [ :anError | anError messageText = 'Owner name is too long' ]! !
